# coding: utf-8
from copy import deepcopy
import datetime
import mongoengine
from django.conf import settings


class ViewTimeLog(mongoengine.Document):
    duration = mongoengine.IntField(help_text='View process duration in seconds')
    view_func_path = mongoengine.StringField()
    view_args = mongoengine.ListField()
    view_kwargs = mongoengine.DictField()
    username = mongoengine.StringField()
    request_get = mongoengine.DictField()
    request_post = mongoengine.DictField()
    dc = mongoengine.DateTimeField()

    meta = {
        'indexes': ['-duration', 'dc'],
        'db_alias': getattr(settings, 'LOG_VIEW_TIME_DB_ALIAS', 'default')
    }

    def save(self, *args, **kwargs):
        if not self.dc:
            self.dc = datetime.datetime.now()
        return super(ViewTimeLog, self).save(*args, **kwargs)


class MysqlSlowQueriesTimeLog(mongoengine.Document):
    start_time = mongoengine.DateTimeField()
    user_host = mongoengine.StringField()
    query_time = mongoengine.IntField()
    lock_time = mongoengine.IntField()
    rows_sent = mongoengine.IntField()
    rows_examined = mongoengine.IntField()
    sql_text = mongoengine.StringField()
    db = mongoengine.StringField()
    last_insert_id = mongoengine.IntField()
    insert_id = mongoengine.IntField()
    server_id = mongoengine.IntField()

    meta = {
        'indexes': ['query_time', 'start_time'],
        'db_alias': getattr(settings, 'LOG_VIEW_TIME_DB_ALIAS', 'default')
    }


class MysqlBinLogTimeLog(mongoengine.Document):
    UPDATE_QUERY_TYPE = 'UPDATE'
    DELETE_QUERY_TYPE = 'DELETE'
    INSERT_QUERY_TYPE = 'INSERT'
    INSERT_INTO_QUERY_TYPE = 'INSERT_INTO'
    SET_QUERY_TYPE = 'SET'
    QUERY_TYPE_CHOICES = (
        (UPDATE_QUERY_TYPE, UPDATE_QUERY_TYPE),
        (DELETE_QUERY_TYPE, DELETE_QUERY_TYPE),
        (INSERT_QUERY_TYPE, INSERT_QUERY_TYPE),
        (INSERT_INTO_QUERY_TYPE, INSERT_INTO_QUERY_TYPE),
        (SET_QUERY_TYPE, SET_QUERY_TYPE),
    )

    LOGGED_QUERY_TYPES = [
        UPDATE_QUERY_TYPE,
        DELETE_QUERY_TYPE,
        INSERT_INTO_QUERY_TYPE,
        INSERT_QUERY_TYPE
    ]

    start_time = mongoengine.DateTimeField()
    exec_time = mongoengine.IntField()
    timestamp = mongoengine.DateTimeField()
    error_code = mongoengine.IntField()
    query = mongoengine.StringField()
    query_type = mongoengine.StringField(choices=QUERY_TYPE_CHOICES)

    meta = {
        'indexes': ['exec_time', 'query_type'],
        'db_alias': getattr(settings, 'LOG_VIEW_TIME_DB_ALIAS', 'default')
    }

class ParsedLogsFiles(mongoengine.Document):
    file_name = mongoengine.StringField(verbose_name='file_name')

    meta = {
        'indexes': ['file_name', ],
        'db_alias': getattr(settings, 'LOG_VIEW_TIME_DB_ALIAS', 'default')
    }

    def get_all_not_parsed_files(self):
        pass



import datetime
from django.conf import settings
from django.contrib.auth.models import User
from djutils.testrunner import TearDownTestCaseMixin
import mock
from django.test import TestCase
from django.test.client import RequestFactory
import models_mongo
from middleware.view_logger import ViewTimeLogger as ViewTimeLoggerMiddleware
import views

class ViewsTestCase(TestCase):
    def setUp(self):
        self.url = '/views_log/'

    def generate_data(self):
        models_mongo.ViewTimeLog.objects.create(
            duration=2,
            view_func_path='time_logger.views.test_view',
            view_args=[1, 2, 3],
            view_kwargs={'a': 1, 'b': 2, 'c': 3},
            username='tester',
            request_get={'q': 'test query'},
            request_post={},
        )

    def test_without_params(self):
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)

    def test_filter_duration(self):
        self.generate_data()

        params = {'min_duration': 1}
        self._assert_exists(params)

        params['min_duration'] = 3
        self._assert_not_exists(params)

    def test_filter_view_func_path(self):
        self.generate_data()
        log = models_mongo.ViewTimeLog.objects.all()[0]

        params = {'view_func_path': log.view_func_path}
        self._assert_exists(params)

        params['view_func_path'] += '123'
        self._assert_not_exists(params)

    def test_filter_dc(self):
        self.generate_data()
        log = models_mongo.ViewTimeLog.objects.all()[0]

        dc_iso = log.dc.isoformat(sep=' ')[:19]
        params = {'min_dc': dc_iso, 'max_dc': dc_iso}
        self._assert_exists(params)

        dc_iso = (log.dc + datetime.timedelta(days=1)).isoformat(sep=' ')[:19]
        params = {'min_dc': dc_iso, 'max_dc': dc_iso}
        self._assert_not_exists(params)

    def _assert_exists(self, params):
        response = self.client.get(self.url, params)
        self.assertTrue(
            len(response.context['page_obj'].object_list)
        )

    def _assert_not_exists(self, params):
        response = self.client.get(self.url, params)
        self.assertFalse(
            len(response.context['page_obj'].object_list)
        )


class ViewMiddlewareTestCase(TestCase, TearDownTestCaseMixin):
    def setUp(self):
        url = '/views_log/'
        factory = RequestFactory()
        self.request = factory.get(url)
        self.middleware = ViewTimeLoggerMiddleware()

    def tearDown(self):
        self.tearDownMongo()

    def test_process_request(self):
        self.middleware.process_request(self.request)

        self.assertTrue(
            hasattr(self.request, 'time_logger')
        )

        self.assertAlmostEqual(
            self.request.time_logger['start_dt'],
            datetime.datetime.now(),
            delta=datetime.timedelta(seconds=1),
        )

    def test_process_view(self):
        view = views.ViewsLog.as_view()
        args = [1, 2, 3]
        kwargs = {'a': 1, 'b': 2, 'c': 3}

        self.middleware.process_request(self.request)
        self.middleware.process_view(self.request, view, args, kwargs)

        self.assertEqual(self.request.time_logger['view_func'], view)
        self.assertEqual(self.request.time_logger['view_args'], args)
        self.assertEqual(self.request.time_logger['view_kwargs'], kwargs)

    @mock.patch('time_logger.middleware.view_logger.ViewTimeLogger._log_view')
    def test_process_response(self, log_view_mock):
        self.middleware.process_response(self.request, None)
        self.assertTrue(log_view_mock.called)

    @mock.patch('time_logger.middleware.view_logger.ViewTimeLogger._log_view')
    def test_process_exception(self, log_view_mock):
        self.middleware.process_exception(self.request, None)
        self.assertTrue(log_view_mock.called)

class LogViewFuncViewMiddlewareTestCase(TestCase, TearDownTestCaseMixin):
    def setUp(self):
        url = '/views_log/'
        factory = RequestFactory()
        self.request = factory.get(url)
        self.middleware = ViewTimeLoggerMiddleware()
        self.request.time_logger = {
            'start_dt': datetime.datetime.now(),
            'view_func': views.ViewsLog.as_view(),
            'view_args': [1, 2, 3],
            'view_kwargs': {'a': 1, 'b': 2, 'c': 3},
        }
        user = User.objects.create(username='tester')
        self.request.user = user

    def tearDown(self):
        self.tearDownMongo()

    def test_no_logging_without_settings(self):
        with self.settings(LOG_VIEW_TIME=None):
            self.middleware._log_view(self.request)
            self.assertFalse(
                models_mongo.ViewTimeLog.objects.all().count()
            )

    def test_no_logging_with_small_duration(self):
        small_delta = datetime.timedelta(seconds=settings.LOG_VIEW_TIME - 2)
        self.request.time_logger['start_dt'] = self.request.time_logger['start_dt'] - small_delta

        self.middleware._log_view(self.request)
        self.assertFalse(
            models_mongo.ViewTimeLog.objects.all().count()
        )

    def test_no_logging_with_big_duration(self):
        big_duration = settings.LOG_VIEW_TIME + 2
        big_delta = datetime.timedelta(seconds=big_duration)
        self.request.time_logger['start_dt'] = self.request.time_logger['start_dt'] - big_delta

        self.middleware._log_view(self.request)
        self.assertTrue(
            models_mongo.ViewTimeLog.objects.all().count()
        )

        log = models_mongo.ViewTimeLog.objects.get()
        self.assertEqual(log.duration, big_duration)
        self.assertEqual(log.view_func_path, 'time_logger.views.ViewsLog')
        self.assertEqual(log.view_args, self.request.time_logger['view_args'])
        self.assertEqual(log.view_kwargs, self.request.time_logger['view_kwargs'])
        self.assertEqual(log.request_get, {})
        self.assertEqual(log.request_post, {})
        self.assertAlmostEqual(log.dc, datetime.datetime.now(), delta=datetime.timedelta(seconds=1))




# coding: utf-8

from django.core.management.base import BaseCommand
from django.db import connection

from time_logger import models_mongo


class Command(BaseCommand):
    help = 'Improt mysql slow query log from mysql to mongodb'

    def handle(self, *args, **options):
        logs = models_mongo.MysqlSlowQueriesTimeLog.objects.all()
        latest_start_time = None
        if logs:
            latest_start_time = logs.order_by('-start_time')[0].start_time
        cursor = connection.cursor()
        cursor.execute("use mysql")

        query = "SELECT * FROM slow_log"
        if latest_start_time:
            query += " WHERE start_time > '%s'" % latest_start_time.isoformat()
        cursor.execute(query)
        fields = map(lambda x: x[0], cursor.description)
        results = [dict(zip(fields, row)) for row in cursor.fetchall()]

        # write results into mongo
        for row in results:
            row['query_time'] = row['query_time'].second
            row['lock_time'] = row['lock_time'].second
            models_mongo.MysqlSlowQueriesTimeLog.objects.create(**row)



# coding: utf-8

from django import forms

class ViewsLoggerForm(forms.Form):
    min_duration = forms.IntegerField(required=False)
    view_func_path = forms.CharField(required=False)
    min_dc = forms.DateTimeField(required=False)
    max_dc = forms.DateTimeField(required=False)


class SlowQueriesLog(forms.Form):
    min_query_time = forms.IntegerField(required=False)
    min_dc = forms.DateTimeField(required=False)
    max_dc = forms.DateTimeField(required=False)


class BinLog(forms.Form):
    min_exec_time = forms.IntegerField(required=False)
    min_dc = forms.DateTimeField(required=False)
    max_dc = forms.DateTimeField(required=False)



# coding: utf-8
import datetime
from django.conf import settings
import inspect

from time_logger import models_mongo

class ViewTimeLogger(object):

    def process_request(self, request):
        request.time_logger = {
            'start_dt': datetime.datetime.now(),
            'view_func': None,
            'view_args': None,
            'view_kwargs': None,
        }
        return None

    def process_view(self, request, view_func, view_args, view_kwargs):
        request.time_logger['view_func'] = view_func
        request.time_logger['view_args'] = view_args
        request.time_logger['view_kwargs'] = view_kwargs

    def process_response(self, request, response):
        self._log_view(request)
        return response

    def process_exception(self, request, exception):
        self._log_view(request)
        return None


    def _log_view(self, request):
        if not hasattr(request, 'time_logger'):
            return

        duration = datetime.datetime.now() - request.time_logger['start_dt']

        is_exceed_time = hasattr(settings, 'LOG_VIEW_TIME') \
                         and settings.LOG_VIEW_TIME \
                         and duration > datetime.timedelta(seconds=settings.LOG_VIEW_TIME)
        if is_exceed_time:
            view_func_module = inspect.getmodule(request.time_logger['view_func'])
            view_func_path = '%s.%s' % (view_func_module.__name__, request.time_logger['view_func'].__name__)

            models_mongo.ViewTimeLog.objects.create(
                duration=duration.seconds,
                view_func_path=view_func_path,
                view_args=request.time_logger['view_args'],
                view_kwargs=request.time_logger['view_kwargs'],
                username=request.user.username,
                request_get=self._query_to_dict(request.GET),
                request_post=self._query_to_dict(request.POST),
            )


    def _query_to_dict(self, qd):
        result = {}
        for key in qd:
            values = qd.getlist(key)
            if len(values) > 1:
                result[key] = values
            else:
                result[key] = qd[key]
        return result




# coding: utf-8
import re
import datetime
import decimal

# not shure in names. So in my logs only Query and Intvar
NOT_PARSING_EVENTS = ['Xid', 'User_var', 'Rand', 'Intvar']

_DATE_PAT = r"\d{6}\s+\d{1,2}:\d{2}:\d{2}"

# SLOW LOG EXPRESSIONS
_SLOW_HEADER_VERSION = re.compile(r"(.+), Version: (\d+)\.(\d+)\.(\d+)(?:-(\S+))?")
_SLOW_HEADER_SERVER = re.compile(r"Tcp port:\s*(\d+)\s+Unix socket:\s+(.*)")

_SLOW_TIMESTAMP = re.compile(r"#\s+Time:\s+(" + _DATE_PAT + r")")
_SLOW_USERHOST = re.compile(r"#\s+User@Host:\s+"
                            r"(?:([\w\d]+))?\s*"
                            r"\[\s*([\w\d]+)\s*\]\s*"
                            r"@\s*"
                            r"([\w\d]*)\s*"
                            r"\[\s*([\d.]*)\s*\]")
_SLOW_STATS = re.compile(r"#\sQuery_time:\s(\d*\.\d{1,6})\s*"
                         r"Lock_time:\s(\d*\.\d{1,6})\s*"
                         r"Rows_sent:\s(\d*)\s*"
                         r"Rows_examined:\s(\d*)")

# BIN_LOG EXPRESSIONS
BIN_LOG_END = '# End of log file'
_BIN_LOG_DELIMITER = re.compile(r"DELIMITER\s(.+);")
_BIN_LOG_QUERY_STATS = re.compile(r"#(" + _DATE_PAT + ")\s+"
                            r"server\s+id\s+(\d+)\s+"
                            r"end_log_pos\s+(\d+)\s+"
                            r"Query\s+thread_id=(\d+)\s+"
                            r"exec_time=(\d+)\s+"
                            r"error_code=(\d+)")
_BIN_LOG_DB = re.compile(r"use `(.+)`")
_BING_LOG_TIMESTAMP = re.compile(r"SET TIMESTAMP=(\d+)")

class LogParserError(Exception):
    pass


class BaseLogParser(object):
    def __iter__(self):
        return self

    def next(self):
        entry = self._parse_entry()
        if entry is None:
            raise StopIteration
        return entry

    def _get_next_line(self):
        line = self._stream.readline()
        if not line:
            return None

        delimiter = getattr(self, 'delimiter', '')

        return line.rstrip('%s\r\n' % delimiter)

    @staticmethod
    def _parse_line(regex, line):
        info = regex.match(line)
        if info is None:
            raise LogParserError('Failed parsing line: %s' %
                                 line[:30])
        return info.groups()

    def _parse_headers(self, line):
        raise NotImplementedError()

    def _parse_entry(self):
        raise NotImplementedError()


class MysqlBinLogParser(BaseLogParser):
    def __init__(self, log_path):
        self._stream = open(log_path, 'r')

        # line which contains query info about new expression
        self._cached_line = None

        line = self._get_next_line()
        if line is not None:
            self._parse_headers(line)

    def _parse_headers(self, line):
        while line and not line.startswith('DELIMITER'):
            line = self._get_next_line()
        self.delimiter = self._parse_line(_BIN_LOG_DELIMITER, line)[0]

        # skip some header info and session
        while not 'BEGIN' in line:
            line = self._get_next_line()
            if line is None:
                return None
        # get first Query
        while not 'Query' in line:
            line = self._get_next_line()
            if line is None:
                return None
        self._cached_line = line

    def _parse_entry(self):
        # this line must contains Query stats
        if self._cached_line:
            line = self._cached_line
        else:
            line = self._get_next_line()

        if line is None:
            return None

        start_time, server_id, end_log_pos, thread_id, exec_time, error_code =\
            self._parse_line(_BIN_LOG_QUERY_STATS, line)

        # str: db
        line = self._get_next_line()
        db = None
        if line.startswith('use'):
            db = self._parse_line(_BIN_LOG_DB, line)[0]
            line = self._get_next_line()

        # str: timestamp
        timestamp = self._parse_line(_BING_LOG_TIMESTAMP, line)[0]

        # str: query
        line = self._get_next_line()
        query_type = line.split(' ')[0]
        query = line

        # skip line to another Query command
        while not 'Query' in line:
            line = self._get_next_line()
            if line == BIN_LOG_END:
                return None
        # cached new query info line
        self._cached_line = line

        return {
            'start_time': start_time,
            'server_id': server_id,
            'end_log_pos': end_log_pos,
            'thread_id': thread_id,
            'exec_time': exec_time,
            'error_code': error_code,
            'db': db,
            'timestamp': timestamp,
            'query': query,
            'query_type': query_type,

        }


class MysqlSlowQueriesParser(BaseLogParser):
    def __init__(self, log_path):
        self._stream = open(log_path, 'r')

        line = self._get_next_line()
        if line is not None and line.endswith('started with:'):
            self._parse_headers(line)

    def _parse_headers(self, line):
        # header strings:
        #   1 run command and version
        #   2 connection info
        #   3 headers
        # we do not need in all this stuff
        self._get_next_line()
        self._get_next_line()
        line = self._get_next_line()
        return line

    def _parse_entry(self):
        entry = {}
        line = self._get_next_line()
        if not line:
            return None
        if line.startswith('# Time:'):
                timestamp = self._parse_time(line)
                entry['start_time'] = timestamp
                line = self._get_next_line()

        if line.startswith('# User@Host:'):
            private_user, unprivate_user, host, ip = self._parse_connection_info(line)
            entry['user'] = private_user if private_user else unprivate_user
            entry['host'] = host if host else ip
            line = self._get_next_line()

        if line.startswith('# Query_time:'):
            query_time, lock_time, rows_sent, rows_examined = self._parse_statistics(line)
            entry['query_time'] = decimal.Decimal(query_time)
            entry['lock_time'] = decimal.Decimal(lock_time)
            entry['rows_sent'] = int(rows_sent)
            entry['rows_examined'] = int(rows_examined)
            line = self._get_next_line()

        queries = self._parse_queries(line)
        entry['queries_list'] = queries

        return entry

    def _parse_time(self, line):
        info = self._parse_line(_SLOW_TIMESTAMP, line)
        return datetime.datetime.strptime(info[0], "%y%m%d %H:%M:%S")

    def _parse_connection_info(self, line):
        try:
            info = self._parse_line(_SLOW_USERHOST, line)
        except:
            info = ('', '', '', '0.0.0.0')
        return info

    def _parse_statistics(self, line):
        return self._parse_line(_SLOW_STATS, line)

    def _parse_queries(self, line):
        query_string = []
        while line:
            if line.startswith('# Time:'):
                break
            query_string.append(line)
            line = self._get_next_line()
        return query_string

# usage example
# if __name__ == '__main__':
#     cnt = 0
#     for log in MysqlBinLogParser('bin_logs.sql'):
#         cnt += 1
#     print cnt



# coding: utf-8

from django.core.management.base import BaseCommand
from time_logger.mysql_logs_parser_from_file import MysqlSlowQueriesParser
from time_logger import models_mongo


class Command(BaseCommand):
    help = 'Improt mysql slow query log from file to mongodb'

    def add_arguments(self, parser):
        parser.add_argument('log_path')

    def handle(self, *args, **options):
        for entry in MysqlSlowQueriesParser(options['log_path']):
            data = {
                'start_time': entry['start_time'],
                'user_host': u'%s@%s' % (entry['user'], entry['host']),
                'query_time': entry['query_time'],
                'lock_time': entry['lock_time'],
                'rows_sent': entry['rows_sent'],
                'sql_text': ' ;'.join(entry['queries_list']),
            }
            models_mongo.MysqlSlowQueriesTimeLog.objects.create(**data)


# coding: utf-8
import os
import datetime

from django.core.management.base import BaseCommand
import subprocess
from time_logger.mysql_logs_parser_from_file import MysqlBinLogParser
from time_logger import models_mongo

PATH_TO_BINLOGS = '/var/log/mysql/'
PATH_TO_READABLE_BINLOGS = '/tmp/binlogs/'

class Command(BaseCommand):
    help = 'Import mysql binlog from file to mongodb'

    def add_arguments(self, parser):
        parser.add_argument('--log_path', default=None)

    def handle(self, *args, **options):
        new_binlog_file_paths = []
        parsed_log_names = models_mongo.ParsedLogsFiles.objects.values_list('file_name')

        log_path = options.get('log_path')
        if log_path:
            file_name = log_path.split('/')[-1]
            if file_name in parsed_log_names:
                raise Exception('file already parsed')

            new_binlog_file_paths.append(log_path)
        else:
            new_binlog_file_names = [
                name for name in os.listdir(PATH_TO_BINLOGS)
                if name not in parsed_log_names and name.startswith('mysql-bin') and name != 'mysql-bin.index'
                ]

            # cutoff last log. Mysql writes binlogs in it
            new_binlog_file_names = sorted(new_binlog_file_names)[:-1]

            # create dir for readable logs
            if not os.path.exists(PATH_TO_READABLE_BINLOGS):
                os.mkdir(PATH_TO_READABLE_BINLOGS)

            # make binlogs readable
            for file_name in new_binlog_file_names:
                # command mysqlbinlog --verbose --base64-output=NEVER  mysql-bin.002491 > bin_logs.sql
                rez = subprocess.call('mysqlbinlog --verbose %s%s > %s%s' %
                                      (PATH_TO_BINLOGS, file_name, PATH_TO_READABLE_BINLOGS, file_name), shell=True)

                new_binlog_file_paths.append('%s%s' % (PATH_TO_READABLE_BINLOGS, file_name))

        for log_path in new_binlog_file_paths:
            for entry in MysqlBinLogParser(log_path):
                if entry['query_type'] in models_mongo.MysqlBinLogTimeLog.LOGGED_QUERY_TYPES:
                    # remove not interested stats
                    del entry['server_id']
                    del entry['end_log_pos']
                    del entry['thread_id']
                    del entry['db']

                    entry['timestamp'] = datetime.datetime.fromtimestamp(entry['timestamp'])
                    models_mongo.MysqlBinLogTimeLog.objects.create(**entry)
                    models_mongo.ParsedLogsFiles.objects.create(file_name=log_path.split('/')[-1])




# coding: utf-8
# python setup.py sdist register upload
from setuptools import setup

setup(
    name='django-time-logger',
    version='0.0.8',
    description='Time logger for django',
    author='Telminov Sergey',
    url='https://github.com/telminov/django-time-logger',
    packages=[
        'time_logger',
        'time_logger/middleware',
        'time_logger/management/commands',
    ],
    include_package_data=True,
    license='The MIT License',
    test_suite='runtests.runtests',
    install_requires=[
        'django',
        'sw-django-utils',
    ],
)



# coding: utf-8
import datetime
from django.views.generic import TemplateView, FormView
from django.views.generic.list import MultipleObjectMixin

from . import forms
from . import models_mongo


class ViewsLog(MultipleObjectMixin, TemplateView):
    form_class = forms.ViewsLoggerForm
    template_name = 'time_logger/views_log.html'
    paginate_by = 30

    def get_context_data(self, **kwargs):
        context = {}

        self.form = self.form_class(self.request.GET or None)
        if self.form.is_valid():
            self.object_list = self.get_queryset()
            context = super(ViewsLog, self).get_context_data(**kwargs)

        context['form'] = self.form
        return context

    def get_queryset(self):
        params = self.get_queryset_params()
        qs = models_mongo.ViewTimeLog.objects.filter(**params)
        return qs

    def get_queryset_params(self):
        params = {}

        if self.form.cleaned_data.get('min_duration'):
            params['duration__gte'] = self.form.cleaned_data['min_duration']

        if self.form.cleaned_data.get('view_func_path'):
            params['view_func_path'] = {
                '$regex': '^%s' % self.form.cleaned_data['view_func_path']
            }

        if self.form.cleaned_data.get('min_dc'):
            params['dc__gte'] = _date_to_datetime(self.form.cleaned_data['min_dc'])

        if self.form.cleaned_data.get('max_dc'):
            params['dc__lte'] = _date_to_datetime_lte(self.form.cleaned_data['max_dc'])

        return params


class SlowQueriesLog(MultipleObjectMixin, TemplateView):
    form_class = forms.SlowQueriesLog
    template_name = 'time_logger/slow_queries_log.html'
    paginate_by = 30

    def get_context_data(self, **kwargs):
        context = {}

        self.form = self.form_class(self.request.GET or None)
        if self.form.is_valid():
            self.object_list = self.get_queryset()
            context = super(SlowQueriesLog, self).get_context_data(**kwargs)

        context['form'] = self.form
        return context

    def get_queryset(self):
        params = self.get_queryset_params()
        qs = models_mongo.MysqlSlowQueriesTimeLog.objects.filter(**params)
        return qs

    def get_queryset_params(self):
        params = {}
        if self.form.cleaned_data.get('min_query_time'):
            params['query_time__gte'] = self.form.cleaned_data['min_query_time']

        if self.form.cleaned_data.get('min_dc'):
            params['start_time__gte'] = _date_to_datetime(self.form.cleaned_data['min_dc'])

        if self.form.cleaned_data.get('max_dc'):
            params['start_time__lte'] = _date_to_datetime_lte(self.form.cleaned_data['max_dc'])

        return params


class BinLog(MultipleObjectMixin, TemplateView):
    form_class = forms.BinLog
    template_name = 'time_logger/bin_log.html'
    paginate_by = 30

    def get_context_data(self, **kwargs):
        context = {}

        self.form = self.form_class(self.request.GET or None)
        if self.form.is_valid():
            self.object_list = self.get_queryset()
            context = super(BinLog, self).get_context_data(**kwargs)

        context['form'] = self.form
        return context

    def get_queryset(self):
        params = self.get_queryset_params()
        qs = models_mongo.MysqlBinLogTimeLog.objects.filter(**params)
        return qs

    def get_queryset_params(self):
        params = {}
        if self.form.cleaned_data.get('min_exec_time'):
            params['exec_time__gte'] = self.form.cleaned_data['min_exec_time']

        if self.form.cleaned_data.get('min_dc'):
            params['start_time__gte'] = _date_to_datetime(self.form.cleaned_data['min_dc'])

        if self.form.cleaned_data.get('max_dc'):
            params['start_time__lte'] = _date_to_datetime_lte(self.form.cleaned_data['max_dc'])

        return params


def _date_to_datetime(date):
    return datetime.datetime(*(date.timetuple()[:6]))


def _date_to_datetime_lte(date):
    return datetime.datetime.combine(date, datetime.time(23, 59, 59))



from django.conf.urls import patterns, url
from time_logger.views import ViewsLog, SlowQueriesLog, BinLog

urlpatterns = patterns('',
   url(r'^views_log/$', ViewsLog.as_view()),
   url(r'^slow_queries_log/$', SlowQueriesLog.as_view()),
   url(r'^bin_log/$', BinLog.as_view()),
)
